import { ControlUnit } from "./Accessories/ControlUnit";
import { DeviceButton } from './Accessories/DeviceButton';
import { IActivity } from "./Models/Config/IActivity";
import { IDeviceSetupItem } from "./Models/Config/IDeviceSetupItem";
import { IInput, IOutput, IMatrix } from "./Models/Config/IMatrix";
import HarmonyDataProvider from "./DataProviders/HarmonyDataProvider";
import { IConfig, IControlUnit } from "./Models/Config/IConfig";
import { IDeviceButton } from "./Models/Config/IDeviceButton";
import { IAccessory } from "./Accessories/IAccessory";
import { IDevice, ICommand } from "./Models/IDevice";

let Accessory: any;
let Homebridge: any;

/**
 * Main entry.
 * @param homebridge 
 */
export default function (homebridge: any) {
    Homebridge = homebridge;
    Accessory = homebridge.platformAccessory;
    homebridge.registerPlatform(
        'homebridge-harmony-watson',
        'HarmonyHubMatrix',
        HarmonyMatrixPlatform,
        true
    );
};

class HarmonyMatrixPlatform {
    log: any = {};
    config: IConfig;
    api: any;
    dataProvider: HarmonyDataProvider | null;
    accessoryList: Array<IAccessory> = [];

    constructor(log: any, config: any, api: any) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.log('INFO - Registering Harmony Matrix Platform');
        this.api.on('didFinishLaunching', this.didFinishLaunching.bind(this));

        this.dataProvider = null;

        if (this.config) {
            //construct data provider
            this.dataProvider = new HarmonyDataProvider({
                hubAddress: this.config.hubIp,
                matrix: this.config.Matrix,
                log: this.log
            });

            //Emit devices if requested
            if (this.config.EmitDevicesOnStartup) {
                this.dataProvider.on("Ready", () => {
                    const devices: { [name: string]: IDevice } = this.dataProvider!.devices;
                    Object.values(devices).forEach((device: IDevice) => {
                        this.log(`${device.name} : ${device.id}`);
                        Object.keys(device.commands).forEach((command: string) => {
                            this.log(`    ${command}`);
                        });
                    });
                });
            }
        }
    }

    /**
     * Handler for didFinishLaunching
     * Happens after constructor
     */
    didFinishLaunching() {
        this.log(`Publishing external accessories`);

        //This is required in order to have multiple tv remotes on one platform
        this.accessoryList.forEach((accessory: IAccessory) => {
            if (accessory instanceof ControlUnit) {
                this.api.publishExternalAccessories("HarmonyMatrixPlatform", [accessory.platformAccessory]);
            }
        })
    }

    /**
     * Called by homebridge to gather accessories.
     * @param callback 
     */
    accessories(callback: (accessories: Array<IAccessory>) => void) {

        //Add control units
        this.config.ControlUnits.forEach((unit: IControlUnit) => {
            this.accessoryList.push(new ControlUnit({
                dataProvider: this.dataProvider!,
                displayName: unit.DisplayName,
                api: this.api,
                log: this.log,
                activities: unit.Activities,
                homebridge: Homebridge,
            }));
        });

        //Add device buttons
        this.config.DeviceButtons.forEach((button: IDeviceButton) => {
            this.accessoryList.push(new DeviceButton({
                dataProvider: this.dataProvider!,
                buttonName: button.ButtonName,
                displayName: button.DisplayName,
                deviceInfo: button,
                api: this.api,
                log: this.log,
                homebridge: Homebridge,

            }))
        });
        callback(this.accessoryList);
    }
}